package tree.data;

import java.util.HashMap;
import java.util.Map;
import java.util.NoSuchElementException;

import tree.model.Constant;
import tree.model.Variable;

/**
 * 
 * Simple factory pattern for creating tree operands :
 *
 */
public class OperandFactory {
	/**
	 * There are two ways for variable reference.
	 */
	private Map<String, Variable> variables = new HashMap<String, Variable>();
	private Map<String, Integer> variableValues = new HashMap<String, Integer>();

	/**
	 * Fly Weight pattern for reusing constant objects.
	 */
	private Map<Integer, Constant> constants = new HashMap<Integer, Constant>();

	/**
	 * Private constructor for Singleton.
	 */
	private OperandFactory() {
		
	}

	/**
	 * Static unique instance for Singleton.
	 */
	private static OperandFactory instance = new OperandFactory();

	public static OperandFactory getInstance() {
		return instance;
	}

	public Variable createVariable(String name, int initialValue) {
		Variable variable = new Variable(name);
		variables.put(name, variable);
		variableValues.put(name, initialValue);
		return variable;
	}

	public Variable getVariable(String name) {
		if (variables.containsKey(name)) {
			return variables.get(name);
		} else {
			throw new NoSuchElementException("No such variable " + name);
		}
	}

	public int getVariableValue(String name) throws NoSuchElementException {
		if (variableValues.containsKey(name)) {
			return variableValues.get(name);
		} else {
			throw new NoSuchElementException("No such variable " + name);
		}
	}

	public Constant createConstant(int value) {
		if (constants.containsKey(value)) {
			return constants.get(value);
		} else {
			Constant constant = new Constant(value);
			constants.put(value, constant);
			return constant;
		}
	}

}
