package tree.facade;

import java.util.List;

import tree.data.TreeBuilder;
import tree.engine.HeigtVisitor;
import tree.engine.SearchConstantVisitor;
import tree.engine.SearchVariableVisitor;
import tree.engine.TreeVisitor;
import tree.model.Constant;
import tree.model.Tree;
import tree.model.Variable;

/**
 * Summary of design patterns used in the whole tree program :
 * 
 * 1) Facade 2) Simple Factory 3) Singleton 4) Fly Weight 5) Composite 6) Visitor 7) Strategy 8) Builder 9) Template Method 10) Interpreter
 * 
 * @author Tianxiao.Liu@u-cergy.fr
 */
public class TreeAppFacade {
	private Tree tree;
	private TreeBuilder treeBuilder;

	private HeigtVisitor heightVisitor = new HeigtVisitor();
	private TreeVisitor<List<Variable>> searchVariableVisitor = new SearchVariableVisitor();
	private TreeVisitor<List<Constant>> searchConstantVisitor = new SearchConstantVisitor();

	public TreeAppFacade(String inputPath) {
		treeBuilder = new TreeBuilder(inputPath);
		tree = treeBuilder.buildTree();
	}

	public String getTreeHeightInfo() {
		heightVisitor.init();
		tree.accept(heightVisitor);
		return "The height of the tree is " + heightVisitor.getHeight() + "\n";

	}

	public String getVariablesInfo() {
		String result = "- All found variables : \n";
		List<Variable> variables = tree.accept(searchVariableVisitor); 
		for (Variable variable : variables) {
			result += "Variable " + variable.getName() + "\n";
		}
		return result;
	}

	public String getConstantsInfo() {
		String result = "- All found constants : \n";
		List<Constant> constants = tree.accept(searchConstantVisitor); 
		for (Constant constant : constants) {
			result += "Constant " + constant.getValue() + "\n";
		}
		return result;
	}

	public String getCalculationInfo() {
		return "The result of the formula " + tree.toString() + " is : " + tree.calculate();
	}

}
